@@ -15,6 +15,7 @@
 import os
 import re
 import sys
+from collections import OrderedDict
 
 # Third party imports
 from qtpy.compat import to_qvariant
@@ -222,18 +223,20 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     zoom_out = Signal()
     zoom_reset = Signal()
     focus_changed = Signal()
+    sig_eol_chars_changed = Signal(str)
     
     def __init__(self, parent=None):
         QPlainTextEdit.__init__(self, parent)
         BaseEditMixin.__init__(self)
         self.setAttribute(Qt.WA_DeleteOnClose)
         
-        self.extra_selections_dict = {}
+        self.extra_selections_dict = OrderedDict()
         
         self.textChanged.connect(self.changed)
         self.cursorPositionChanged.connect(self.cursor_position_changed)
         
         self.indent_chars = " "*4
+        self.tab_stop_width_spaces = 4
         
         # Code completion / calltips
         if parent is not None:
@@ -252,7 +255,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.completion_text = ""
         self.setup_completion()
 
-        self.calltip_widget = CallTipWidget(self, hide_timer_on=True)
+        self.calltip_widget = CallTipWidget(self, hide_timer_on=False)
         self.calltips = True
         self.calltip_position = None
 
@@ -277,6 +280,11 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     def set_indent_chars(self, indent_chars):
         self.indent_chars = indent_chars
 
+    def set_tab_stop_width_spaces(self, tab_stop_width_spaces):
+        self.tab_stop_width_spaces = tab_stop_width_spaces
+        self.setTabStopWidth(tab_stop_width_spaces
+                             * self.fontMetrics().width('9'))
+
     def set_palette(self, background, foreground):
         """
         Set text editor palette colors:
@@ -298,28 +306,40 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
 
     #------Extra selections
+    def extra_selection_length(self, key):
+        selection = self.get_extra_selections(key)
+        if selection:
+            cursor = self.extra_selections_dict[key][0].cursor
+            selection_length = cursor.selectionEnd() - cursor.selectionStart()
+            return selection_length
+        else:
+            return 0
+
     def get_extra_selections(self, key):
         return self.extra_selections_dict.get(key, [])
 
     def set_extra_selections(self, key, extra_selections):
         self.extra_selections_dict[key] = extra_selections
-        
+        self.extra_selections_dict = \
+            OrderedDict(sorted(self.extra_selections_dict.items(),
+                               key=lambda s: self.extra_selection_length(s[0]),
+                               reverse=True))
+
     def update_extra_selections(self):
         extra_selections = []
         for key, extra in list(self.extra_selections_dict.items()):
             if key == 'current_line' or key == 'current_cell':
-                # Python 3 compatibility (weird): current line has to be 
+                # Python 3 compatibility (weird): current line has to be
                 # highlighted first
                 extra_selections = extra + extra_selections
             else:
                 extra_selections += extra
         self.setExtraSelections(extra_selections)
-        
+
     def clear_extra_selections(self, key):
         self.extra_selections_dict[key] = []
         self.update_extra_selections()
-        
-        
+
     def changed(self):
         """Emit changed signal"""
         self.modificationChanged.emit(self.document().isModified())
@@ -609,6 +629,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             self.setTextCursor(cursor)
         text = self.get_selection_as_executable_code()
         self.__restore_selection(start_pos, end_pos)
+        if text is not None:
+            text = text.rstrip()
         return text
 
     def is_cell_separator(self, cursor=None, block=None):
@@ -860,6 +882,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
         if after_current_line:
             text = to_text_string(cursor.block().text())
+            if len(text) == 0:
+                #If the next line is blank
+                sel_text = sel_text[0:-1]
+                sel_text = os.linesep + sel_text
             if not text:
                 cursor.insertText(sel_text)
                 cursor.endEditBlock()
